Performance testing server
==========================

### Steps to deploy
1. Choose an appropriate namespace `<namespace>` to use as Kubernetes namespace
   and to identify a Docker repository (see below for more on this).
2. Choose a tag `<tag>` to identify this specific flavour of the server. It is
   used to name the produced Docker image as well as to name the release of the
   Helm chart, and to construct the DNS name through which the server can be
   reached (see below).
3. Choose `<base image>` as the tag of the Docker image on which to base the new
   Docker image. This is one of the tags available in Docker repository
   `012067661104.dkr.ecr.eu-west-1.amazonaws.com/ci/mongodb-realm-images`. If
   you don't know which tag to use, use the latest one, or just use
   `test_server-859016c167fda99bf492e53ef13322b95a9462b9-race`.
4. Build a Docker image with the test server by running `sh build.sh <namespace>
   <tag> <base image>`. This requires access to Docker registry (see below).
5. Deploy the test server into Arena Kubernetes cluster by running `sh deploy.sh
   <namespace> <tag>` (see below for how to get access).

For example, if the namespace is `xx`, the tag is `foo`, and the base image tag
is `test_server-859016c167fda99bf492e53ef13322b95a9462b9-race`, here are the
commands needed to build and deploy the server:

    sh build.sh xx foo test_server-859016c167fda99bf492e53ef13322b95a9462b9-race
    sh deploy.sh xx foo

The specified namespace, `<namespace>`, must be one of the existing Kubernetes
namespaces in the Arena cluster (see below on how to get access). You may need
to create a new namespace there (talk to Adam Lebsack). The namespace is also
used to name a unique Docker repository on Amazon Web Services. The Docker
repository path is
`012067661104.dkr.ecr.eu-west-1.amazonaws.com/test/sync-perf-<namespace>-server`,
which you will need to create (talk to Adam Lebsack), if it does not already
exist.

When deployed, the server will be reachable within the Kubernetes namespace as
`sync-perf-test-server-<tag>` on port 9090. From outside the Arena cluster, it
will be available as
`sync-perf-test-<namespace>-server-<tag>.arena.realmlab.net` on port 9090. But
please note that it sometimes takes quite a long time (20 minutes, or more) for
external DNS to update.

Rebuilding the Docker image requires access to the Docker registry at
`012067661104.dkr.ecr.eu-west-1.amazonaws.com`. To gain access, first configure
the AWS client using `aws configure` (set region to `eu-west-1`), then run the
Docker login command that is generated by running `aws ecr get-login
--registry-ids 012067661104`.

To gain access to the Arena cluster from your command line, first go to the
[Kubernetes web interface](https://arena.k8s.realmlab.net), and login. Then
update your local environment (`~/.kube/config`) by running the script displayed
on [this page](https://arena.k8s.realmlab.net/kubeconfig).

### Scripts

| Name      | Description
|-----------|----------------------------------------------------------------------------
| build.sh  | Rebuild and publish the Docker image containing the server (`docker build`)
| deploy.sh | Deploy the server into Arena cluster (`helm upgrade`)
| status.sh | See status on server deployment (`helm status`)
| logs.sh   | Show log output from deployed server (`kubectl logs`)
| purge.sh  | Delete server deployment (`helm delete`)
| list.sh   | List all current server deployments in namespace (`helm list`)

These scripts have been tested with [Helm
2.16.7](https://github.com/helm/helm/releases/tag/v2.16.7).
